@@ -7,12 +7,13 @@ Created on Sat Oct 24 14:38:58 2020
 """
 
 __version__ = '0.5.2'
-__release__ = 20230107
-__all__ = ['unit_product', 'unit_division', 'unit_base_power']
+__release__ = 20230116
+__all__ = ['unit_product', 'unit_division', 'unit_base_power', 'unit_power', 'unit_addition', 'unit_inverse']
 
-from unyts.dictionaries import dictionary
+from unyts.dictionaries import dictionary, unitless_names
 from unyts.converter import convertible
 from unyts.helpers.is_number import is_number
+from unyts.helpers.unit_string_tools import reduce_units
 from unyts.helpers.multi_split import multi_split
 
 
@@ -26,18 +27,31 @@ def unit_base_power(unit_string: str) -> tuple:  # tuple[str, int]
     oth = ''
     inv = False
     if '/' in unit_string and len(unit_string.split('/')) == 2 and is_number(unit_string.split('/')[0]):
-        inv_pow, unit_string = unit_string.split('/')
+        inv_pow, u_bas = unit_string.split('/')
         inv = True
-
-    for c in unit_string:
-        if c.isdigit():
-            u_pow += oth + c
-            oth = ''
-        elif c in ['-', '+', '.']:
-            oth += c
-        else:
-            u_bas += oth + c
-            oth = ''
+    else:
+        for c in unit_string:
+            if c.isdigit():
+                u_pow += oth + c
+                oth = ''
+            elif c in ['-', '+', '.']:
+                oth += c
+            else:
+                if inv and len(u_pow) > 0:
+                    if u_pow[0] == '-':
+                        u_pow = u_pow[1:]
+                    else:
+                        u_pow = '-' + u_pow
+                if c in ['/']:
+                    inv = not inv
+                u_bas += oth + c
+                oth = ''
+        if inv and len(u_pow) > 0:
+            if u_pow[0] == '-':
+                u_pow = u_pow[1:]
+            else:
+                u_pow = '-' + u_pow
+        inv = False
     u_pow = 1 if u_pow == '' else float(u_pow) if '.' in u_pow else int(u_pow)
     if inv:
         u_pow = u_pow * -1 * (float(inv_pow) if '.' in inv_pow else int(inv_pow))
@@ -66,10 +80,10 @@ def unit_product(unit_string1: str, unit_string2: str) -> str:
             unit_string2.split('/')[1]:
         unit_string2 = 'dimensionless'
 
-    if unit_string2.lower().strip(' ()') in dictionary['Dimensionless']:
+    if unit_string2.lower().strip(' ()') in unitless_names:  # dictionary['Dimensionless']:
         return unit_string1
-    if unit_string1.lower().strip(' ()') in dictionary['Dimensionless']:
-        if unit_string2.lower().strip(' ()') not in dictionary['Dimensionless']:
+    if unit_string1.lower().strip(' ()') in unitless_names:  # dictionary['Dimensionless']:
+        if unit_string2.lower().strip(' ()') not in unitless_names:  # dictionary['Dimensionless']:
             return unit_string2
         else:
             return unit_string1
@@ -80,6 +94,11 @@ def unit_product(unit_string1: str, unit_string2: str) -> str:
     u1bas, u1pow = unit_base_power(unit_string1)
     u2bas, u2pow = unit_base_power(unit_string2)
 
+    if u1pow == -1 and u2pow != -1:
+        return unit_division(unit_string2, u1bas)
+    elif u1pow != -1 and u2pow == -1:
+        return unit_division(unit_string1, u2bas)
+
     if convertible(u1bas, u2bas):
         u_pow = u1pow + u2pow
         if u_pow == -1:
@@ -104,7 +123,7 @@ def unit_product(unit_string1: str, unit_string2: str) -> str:
                 unit_string2 = '(' + unit_string2 + ')'
                 break
         result = unit_string1 + '*' + unit_string2
-    return result
+    return reduce_units(result, raise_error=False)
 
 
 def unit_division(unit_string1: str, unit_string2: str) -> str:
@@ -125,10 +144,10 @@ def unit_division(unit_string1: str, unit_string2: str) -> str:
             unit_string2.split('/')[1]:
         unit_string2 = 'dimensionless'
 
-    if unit_string2.lower().strip(' ()') in dictionary['Dimensionless']:
+    if unit_string2.lower().strip(' ()') in unitless_names:  # dictionary['Dimensionless']:
         return unit_string1
-    if unit_string1.lower().strip(' ()') in dictionary['Dimensionless']:
-        if unit_string2.lower().strip(' ()') not in dictionary['Dimensionless']:
+    if unit_string1.lower().strip(' ()') in unitless_names:  # dictionary['Dimensionless']:
+        if unit_string2.lower().strip(' ()') not in unitless_names:  # dictionary['Dimensionless']:
             u_bas, u_pow = unit_base_power('1/' + unit_string2)
             return u_bas + str(u_pow)
         else:
@@ -140,6 +159,11 @@ def unit_division(unit_string1: str, unit_string2: str) -> str:
     u1bas, u1pow = unit_base_power(unit_string1)
     u2bas, u2pow = unit_base_power(unit_string2)
 
+    if u1pow == -1 and u2pow != -1:
+        return unit_product(u1bas, unit_string2)
+    elif u1pow != -1 and u2pow == -1:
+        return unit_product(unit_string1, u2bas)
+
     if convertible(u1bas, u2bas):
         u_pow = u1pow - u2pow
         if u_pow == -1:
@@ -176,7 +200,7 @@ def unit_division(unit_string1: str, unit_string2: str) -> str:
                 unit_string2 = '(' + unit_string2 + ')'
                 break
         result = unit_string1 + '/' + unit_string2
-    return result
+    return reduce_units(result, raise_error=False)
 
 
 def unit_power(unit_string: str, power: int or str) -> str:
@@ -211,3 +235,53 @@ def unit_power(unit_string: str, power: int or str) -> str:
             return u1bas + '^' + str(u_pow) + u2bas
     else:
         raise TypeError('Power must be units string or numeric.')
+
+
+def unit_addition(unit_string1:str, unit_string2:str) -> str:
+    if unit_string1 is None:
+        unit_string1 = 'dimensionless'
+    if unit_string2 is None:
+        unit_string2 = 'dimensionless'
+
+    if unit_string2.lower().strip(' ()') in unitless_names:  # dictionary['Dimensionless']:
+        return unit_string1
+    if unit_string1.lower().strip(' ()') in unitless_names:  # dictionary['Dimensionless']:
+        if unit_string2.lower().strip(' ()') not in unitless_names:  # dictionary['Dimensionless']:
+            return unit_string2
+        else:
+            return unit_string1
+
+    if convertible(unit_string2, unit_string1):
+        return unit_string1
+    elif convertible(unit_string1, unit_string2):
+        return unit_string2
+    else:
+        return unit_string1 + '+' + unit_string2
+
+
+def unit_inverse(unit_string: str) -> str:
+    if unit_string is None:
+        unit_string = 'dimensionless'
+
+    if type(unit_string) is str and len(unit_string.strip(' ()')) == 0:
+        unit_string = 'dimensionless'
+
+    if unit_string.lower().strip(' ()') in unitless_names:  # dictionary['Dimensionless']:
+        return unit_string
+
+    ubas, upow = unit_base_power(unit_string)
+    if upow == -1:
+        return ubas
+    elif upow < -1:
+        return ubas + str(abs(upow))
+    elif upow > 1:
+        return ubas + str(-1 * upow)
+
+    if '/' not in unit_string:
+        return ubas + '-1'  # '1/' + ubas
+    elif len(unit_string.split('/')) == 2:
+        return unit_string.split('/')[1] + '/' + unit_string.split('/')[0]
+    elif len(unit_string.split('/')) == 3:
+        return unit_string.split('/')[2] + '*' + unit_string.split('/')[1] + '/' + unit_string.split('/')[0]
+    else:  # wtf?
+        return '/'.join(unit_string.split('/')[::-1])
