@@ -6,9 +6,9 @@ Created on Sat Oct 24 18:24:20 2020
 @author: Martín Carlos Araya <martinaraya@gmail.com>
 """
 
-__version__ = '0.5.1'
-__release__ = 20230106
-__all__ = ['unyts_parameters_', 'print_path', 'reload', 'raise_error', 'cache']
+__version__ = '0.5.2'
+__release__ = 20230107
+__all__ = ['unyts_parameters_', 'print_path', 'reload', 'raise_error', 'cache', 'dir_path']
 
 import os.path
 from json import load as json_load, dump as json_dump
