@@ -6,13 +6,14 @@ Created on Sat Oct 24 14:34:59 2020
 @author: Martín Carlos Araya <martinaraya@gmail.com>
 """
 
-__version__ = '0.5.4'
-__release__ = 20230112
+__version__ = '0.5.7'
+__release__ = 20230117
 __all__ = ['Unit', 'is_Unit']
 
 from unyts.errors import WrongUnitsError, WrongValueError, NoConversionFoundError
-from unyts.operations import unit_product, unit_division, unit_base_power
-from unyts.converter import convert, convertible
+from unyts.operations import unit_product as _unit_product, unit_division as _unit_division, unit_base_power as _unit_base_power
+from unyts.helpers.unit_string_tools import reduce_units as _reduce_units
+from unyts.converter import convert as _convert, convertible as _convertible
 from numbers import Number
 
 
@@ -74,13 +75,27 @@ class Unit(object):
     def __str__(self) -> str:
         return str(self.value) + '_' + str(self.unit)
 
+    @property
+    def dtype(self):
+        if hasattr(self.values, 'dtype'):
+            return self.values.dtype
+        elif _numpy_:
+            if type(self.value) is int:
+                return np.dtype(int)
+            elif type(self.value) is float:
+                return np.dtype(float)
+            elif type(self.value) is complex:
+                return np.dtype(complex)
+        else:
+            raise NotImplementedError("dtype not implemented without NumPy.")
+
     def convert(self, new_unit):
         if type(new_unit) is not str:
             try:
                 new_unit = new_unit.unit
             except AttributeError:
                 raise WrongUnitsError("'" + str(new_unit) + "' for '" + str(self.name) + "'")
-        return self.kind(convert(self.value, self.unit, new_unit), new_unit)
+        return self.kind(_convert(self.value, self.unit, new_unit), new_unit)
 
     def to(self, new_unit):
         return self.convert(new_unit)
@@ -104,10 +119,10 @@ class Unit(object):
             if other.kind is self.kind:
                 if self.unit == other.unit:
                     return self.kind(self.value + other.value, self.unit)
-                elif convertible(other.unit, self.unit):
-                    return self.kind(self.value + convert(other.value, other.unit, self.unit), self.unit)
-                elif convertible(self.unit, other.unit):
-                    return self.kind(other.value + convert(self.value, self.unit, other.unit), other.unit)
+                elif _convertible(other.unit, self.unit):
+                    return self.kind(self.value + _convert(other.value, other.unit, self.unit), self.unit)
+                elif _convertible(self.unit, other.unit):
+                    return self.kind(other.value + _convert(self.value, self.unit, other.unit), other.unit)
                 else:
                     raise NoConversionFoundError("from '" + self.unit + "' to '" + other.unit + "'")
             if self.kind in (Dimensionless, Percentage) and other.kind not in (Dimensionless, Percentage):
@@ -131,10 +146,10 @@ class Unit(object):
             return self.kind((self.value + other) * 100, self.unit)
         elif type(other) in numeric:
             return self.kind(self.value + other, self.unit)
-        elif hasattr(other, '_identity_') and other._identity_ in ('SimSeries', 'SimDataFrame'):
+        elif hasattr(other, 'type') and other.type in ('SimSeries', 'SimDataFrame'):
             return other.__radd__(self)
         else:
-            raise NotImplementedError("Addition of Unit and " + type(other) + " not implemented.")
+            raise NotImplementedError("Addition of Unit and " + str(type(other)) + " not implemented.")
 
     def __radd__(self, other):
         return self.__add__(other)
@@ -150,18 +165,18 @@ class Unit(object):
             elif other.kind in (Dimensionless, Percentage):
                 return self.kind(self.value * other.value, self.unit)
             elif self.unit == other.unit:
-                return units(self.value * other.value, unit_product(self.unit, other.unit))
-            elif convertible(other.unit, self.unit):
-                return units(self.value * convert(other.value, other.unit, self.unit),
-                             unit_product(self.unit, other.unit))
-            elif convertible(self.unit, other.unit):
-                return units(other.value * convert(self.value, self.unit, other.unit),
-                             unit_product(other.unit, self.unit))
-            elif convertible(unit_base_power(self.unit)[0], unit_base_power(other.unit)[0]):
-                factor = convert(1, unit_base_power(other.unit)[0], unit_base_power(self.unit)[0])
-                return units(self.value * other.value * factor, unit_product(self.unit, other.unit))
+                return units(self.value * other.value, _unit_product(self.unit, other.unit))
+            elif _convertible(other.unit, self.unit):
+                return units(self.value * _convert(other.value, other.unit, self.unit),
+                             _unit_product(self.unit, other.unit))
+            elif _convertible(self.unit, other.unit):
+                return units(other.value * _convert(self.value, self.unit, other.unit),
+                             _unit_product(other.unit, self.unit))
+            elif _convertible(_unit_base_power(self.unit)[0], _unit_base_power(other.unit)[0]):
+                factor = _convert(1, _unit_base_power(other.unit)[0], _unit_base_power(self.unit)[0])
+                return units(self.value * other.value * factor, _unit_product(self.unit, other.unit))
             else:
-                return units(self.value * other.value, unit_product(self.unit, other.unit))
+                return units(self.value * other.value, _unit_product(self.unit, other.unit))
         elif type(other) is tuple:
             result = []
             for each in other:
@@ -171,7 +186,7 @@ class Unit(object):
             return self.kind(self.value * other * 100, self.unit)
         elif type(other) in numeric:
             return self.kind(self.value * other, self.unit)
-        elif hasattr(other, '_identity_') and other._identity_ in ('SimSeries', 'SimDataFrame'):
+        elif hasattr(other, 'type') and other.type in ('SimSeries', 'SimDataFrame'):
             return other.__rmul__(self)
         else:
             raise NotImplementedError("Product of Unit and " + type(other) + " not implemented.")
@@ -191,10 +206,10 @@ class Unit(object):
                 return self.kind(self.value ** other.value, self.unit)
             elif self.unit == other.unit:
                 return units(self.value ** other.value, self.unit + '^' + other.unit)
-            elif convertible(other.unit, self.unit):
-                return units(self.value ** convert(other.value, other.unit, self.unit), self.unit + '^' + self.unit)
-            elif convertible(unit_base_power(self.unit)[0], unit_base_power(other.unit)[0]):
-                factor = convert(1, unit_base_power(other.unit)[0], unit_base_power(self.unit)[0])
+            elif _convertible(other.unit, self.unit):
+                return units(self.value ** _convert(other.value, other.unit, self.unit), self.unit + '^' + self.unit)
+            elif _convertible(_unit_base_power(self.unit)[0], _unit_base_power(other.unit)[0]):
+                factor = _convert(1, _unit_base_power(other.unit)[0], _unit_base_power(self.unit)[0])
                 return units(self.value ** (other.value * factor), self.unit + '^' + other.unit)
             else:
                 return units(self.value ** other.value, self.unit + '^' + other.unit)
@@ -206,15 +221,15 @@ class Unit(object):
         elif self.kind is Percentage:
             return self.kind((self.value ** other) * 100, self.unit)
         elif type(other) in numeric:
-            pow_units = unit_base_power(self.unit)[1] * other
+            pow_units = _unit_base_power(self.unit)[1] * other
             if pow_units == 0:
                 pow_units = 'Dimensionless'
             elif pow_units == 1:
-                pow_units = unit_base_power(self.unit)[0]
+                pow_units = _unit_base_power(self.unit)[0]
             else:
-                pow_units = unit_base_power(self.unit)[0] + str(pow_units)
+                pow_units = _unit_base_power(self.unit)[0] + str(pow_units)
             return units(self.value ** other, pow_units)
-        elif hasattr(other, '_identity_') and other._identity_ in ('SimSeries', 'SimDataFrame'):
+        elif hasattr(other, 'type') and other.type in ('SimSeries', 'SimDataFrame'):
             return other.__rpow__(self)
         else:
             raise NotImplementedError("Power of Unit to " + type(other) + " not implemented.")
@@ -236,10 +251,10 @@ class Unit(object):
             if other.kind is self.kind:
                 if self.unit == other.unit:
                     return self.kind(self.value - other.value, self.unit)
-                elif convertible(other.unit, self.unit):
-                    return self.kind(self.value - convert(other.value, other.unit, self.unit), self.unit)
-                elif convertible(self.unit, other.unit):
-                    return self.kind(other.value - convert(self.value, self.unit, other.unit), other.unit)
+                elif _convertible(other.unit, self.unit):
+                    return self.kind(self.value - _convert(other.value, other.unit, self.unit), self.unit)
+                elif _convertible(self.unit, other.unit):
+                    return self.kind(other.value - _convert(self.value, self.unit, other.unit), other.unit)
                 else:
                     raise NoConversionFoundError("from '" + self.unit + "' to '" + other.unit + "'")
             if self.kind in (Dimensionless, Percentage) and other.kind not in (Dimensionless, Percentage):
@@ -263,7 +278,7 @@ class Unit(object):
             return self.kind((self.value - other) * 100, self.unit)
         elif type(other) in numeric:
             return self.kind(self.value - other, self.unit)
-        elif hasattr(other, '_identity_') and other._identity_ in ('SimSeries', 'SimDataFrame'):
+        elif hasattr(other, 'type') and other.type in ('SimSeries', 'SimDataFrame'):
             return other.__rsub__(self)
         else:
             raise NotImplementedError("Subtraction of Unit and " + type(other) + " not implemented.")
@@ -282,15 +297,15 @@ class Unit(object):
             elif other.kind in (Dimensionless, Percentage):
                 return self.kind(self.value / other.value, self.unit)
             elif self.unit == other.unit:
-                return units(self.value / other.value, unit_division(self.unit, other.unit))
-            elif convertible(other.unit, self.unit):
-                return units(self.value / convert(other.value, other.unit, self.unit),
-                             unit_division(self.unit, other.unit))
-            elif convertible(unit_base_power(self.unit)[0], unit_base_power(other.unit)[0]):
-                factor = convert(1, unit_base_power(other.unit)[0], unit_base_power(self.unit)[0])
-                return units(self.value / (other.value * factor), unit_division(self.unit, other.unit))
+                return units(self.value / other.value, _unit_division(self.unit, other.unit))
+            elif _convertible(other.unit, self.unit):
+                return units(self.value / _convert(other.value, other.unit, self.unit),
+                             _unit_division(self.unit, other.unit))
+            elif _convertible(_unit_base_power(self.unit)[0], _unit_base_power(other.unit)[0]):
+                factor = _convert(1, _unit_base_power(other.unit)[0], _unit_base_power(self.unit)[0])
+                return units(self.value / (other.value * factor), _unit_division(self.unit, other.unit))
             else:
-                return units(self.value / other.value, unit_division(self.unit, other.unit))
+                return units(self.value / other.value, _unit_division(self.unit, other.unit))
         elif type(other) is tuple:
             result = []
             for each in other:
@@ -300,7 +315,7 @@ class Unit(object):
             return self.kind(self.value / other * 100, self.unit)
         elif type(other) in numeric:
             return units(self.value / other, self.unit)
-        elif hasattr(other, '_identity_') and other._identity_ in ('SimSeries', 'SimDataFrame'):
+        elif hasattr(other, 'type') and other.type in ('SimSeries', 'SimDataFrame'):
             return other.__rtruediv__(self)
         else:
             raise NotImplementedError("Division of Unit by " + type(other) + " not implemented.")
@@ -328,15 +343,15 @@ class Unit(object):
             elif other.kind in (Dimensionless, Percentage):
                 return self.kind(self.value // other.value, self.unit)
             elif self.unit == other.unit:
-                return units(self.value // other.value, unit_division(self.unit, other.unit))
-            elif convertible(other.unit, self.unit):
-                return units(self.value // convert(other.value, other.unit, self.unit),
-                             unit_division(self.unit, other.unit))
-            elif convertible(unit_base_power(self.unit)[0], unit_base_power(other.unit)[0]):
-                factor = convert(1, unit_base_power(other.unit)[0], unit_base_power(self.unit)[0])
-                return units(self.value // (other.value * factor), unit_division(self.unit, other.unit))
+                return units(self.value // other.value, _unit_division(self.unit, other.unit))
+            elif _convertible(other.unit, self.unit):
+                return units(self.value // _convert(other.value, other.unit, self.unit),
+                             _unit_division(self.unit, other.unit))
+            elif _convertible(_unit_base_power(self.unit)[0], _unit_base_power(other.unit)[0]):
+                factor = _convert(1, _unit_base_power(other.unit)[0], _unit_base_power(self.unit)[0])
+                return units(self.value // (other.value * factor), _unit_division(self.unit, other.unit))
             else:
-                return units(self.value // other.value, unit_division(self.unit, other.unit))
+                return units(self.value // other.value, _unit_division(self.unit, other.unit))
         elif type(other) is tuple:
             result = []
             for each in other:
@@ -346,8 +361,8 @@ class Unit(object):
             return self.kind(self.value // other * 100, self.unit)
         elif type(other) in numeric:
             return self.kind(self.value // other, self.unit)
-        elif hasattr(other, '_identity_') and other._identity_ in ('SimSeries', 'SimDataFrame'):
-            return other.__rtruediv__(self)
+        elif hasattr(other, 'type') and other.type in ('SimSeries', 'SimDataFrame'):
+            return other.__rfloordiv__(self)
         else:
             raise NotImplementedError("Division of Unit by " + type(other) + " not implemented.")
 
@@ -363,6 +378,32 @@ class Unit(object):
         else:
             raise NotImplementedError("Division of Unit by " + type(other) + " not implemented.")
 
+    def __matmul__(self, other):
+        from .units.unitless import Dimensionless, Percentage
+        from .units.define import units
+        if '.units.' in str(type(other)):
+            if self.kind in (Dimensionless, Percentage) and other.kind not in (Dimensionless, Percentage):
+                return other.kind(self.value / other.value, '1/' + other.unit)
+            elif self.kind is Percentage and other.kind in (Dimensionless, Percentage):
+                return self.kind((self.value / other.value) * 100, self.unit)
+            elif other.kind in (Dimensionless, Percentage):
+                return self.kind(self.value / other.value, self.unit)
+            else:
+                return units(self.value / other.value, _reduce_units(self.unit + '/' + other.unit))
+        elif type(other) is tuple:
+            result = []
+            for each in other:
+                result += [self / each]
+            return tuple(result)
+        elif self.kind is Percentage:
+            return self.kind(self.value / other * 100, self.unit)
+        elif type(other) in numeric:
+            return self.kind(self.value / other, self.unit)
+        elif hasattr(other, 'type') and other.type in ('SimSeries', 'SimDataFrame'):
+            return other.__rmatmul__(self)
+        else:
+            raise NotImplementedError("Division of Unit by " + type(other) + " not implemented.")
+
     def __mod__(self, other):
         from .units.unitless import Dimensionless, Percentage
         from .units.define import units
@@ -375,10 +416,10 @@ class Unit(object):
                 return self.kind(self.value % other.value, self.unit)
             elif self.unit == other.unit:
                 return units(self.value % other.value, self.unit)
-            elif convertible(other.unit, self.unit):
-                return units(self.value % convert(other.value, other.unit, self.unit), self.unit)
-            elif convertible(unit_base_power(self.unit)[0], unit_base_power(other.unit)[0]):
-                factor = convert(1, unit_base_power(other.unit)[0], unit_base_power(self.unit)[0])
+            elif _convertible(other.unit, self.unit):
+                return units(self.value % _convert(other.value, other.unit, self.unit), self.unit)
+            elif _convertible(_unit_base_power(self.unit)[0], _unit_base_power(other.unit)[0]):
+                factor = _convert(1, _unit_base_power(other.unit)[0], _unit_base_power(self.unit)[0])
                 return units(self.value % (other.value * factor), self.unit)
             else:
                 return units(self.value % other.value, self.unit)
@@ -391,8 +432,8 @@ class Unit(object):
             return self.kind(self.value % other * 100, self.unit)
         elif type(other) in numeric:
             return self.kind(self.value % other, self.unit)
-        elif hasattr(other, '_identity_') and other._identity_ in ('SimSeries', 'SimDataFrame'):
-            return other.__rtruediv__(self)
+        elif hasattr(other, 'type') and other.type in ('SimSeries', 'SimDataFrame'):
+            return other.__rmod__(self)
         else:
             raise NotImplementedError("Module of Unit when divided by " + type(other) + " not implemented.")
 
@@ -425,7 +466,7 @@ class Unit(object):
             raise TypeError(msg)
 
     def __eq__(self, other) -> bool:
-        if type(self) == type(other):
+        if type(self) is type(other):
             return self.value == other.convert(self.unit).value
         else:
             msg = "'==' not supported between instances of '" + \
@@ -439,7 +480,7 @@ class Unit(object):
             raise TypeError(msg)
 
     def __ne__(self, other):
-        if type(self) == type(other):
+        if type(self) is type(other):
             return self.value != other.convert(self.unit).value
         else:
             msg = "'!=' not supported between instances of '" + \
@@ -453,7 +494,7 @@ class Unit(object):
             raise TypeError(msg)
 
     def __ge__(self, other):
-        if type(self) == type(other):
+        if type(self) is type(other):
             return self.value >= other.convert(self.unit).value
         else:
             msg = "'>=' not supported between instances of '" + \
@@ -467,7 +508,7 @@ class Unit(object):
             raise TypeError(msg)
 
     def __gt__(self, other):
-        if type(self) == type(other):
+        if type(self) is type(other):
             return self.value > other.convert(self.unit).value
         else:
             msg = "'>' not supported between instances of '" + \
@@ -486,6 +527,9 @@ class Unit(object):
         except TypeError:
             return 1
 
+    def __round__(self, n=None):
+        return self.kind(round(self.value, n), self.unit)
+
     def __getitem__(self, item):
         if type(item) is int:
             if item >= len(self):
