@@ -6,8 +6,8 @@ Created on Sat Oct 24 14:34:59 2020
 @author: Martín Carlos Araya <martinaraya@gmail.com>
 """
 
-__version__ = '0.4.9'
-__release__ = 20221231
+__version__ = '0.5.2'
+__release__ = 20230107
 __all__ = ['Pressure', 'Weight', 'Compressibility']
 
 from unyts.dictionaries import dictionary
@@ -18,7 +18,7 @@ from unyts.helpers.common_classes import unit_or_str, numeric
 class Pressure(Unit):
     classUnits = dictionary['Pressure']
 
-    def __init__(self, value: numeric, units: unit_or_str) -> Unit:
+    def __init__(self, value: numeric, units: unit_or_str):
         self.name = 'pressure'
         self.kind = Pressure
         self.value = self.check_value(value)
@@ -28,7 +28,7 @@ class Pressure(Unit):
 class Weight(Unit):
     classUnits = dictionary['Weight']
 
-    def __init__(self, value: numeric, units: unit_or_str) -> Unit:
+    def __init__(self, value: numeric, units: unit_or_str):
         self.name = 'weight'
         self.kind = Weight
         self.value = self.check_value(value)
@@ -38,7 +38,7 @@ class Weight(Unit):
 class Compressibility(Unit):
     classUnits = dictionary['Compressibility']
 
-    def __init__(self, value: numeric, units: unit_or_str) -> Unit:
+    def __init__(self, value: numeric, units: unit_or_str):
         self.name = 'compressibility'
         self.kind = Compressibility
         self.value = self.check_value(value)
